#ifndef KADDRESSBOOK_H
#define KADDRESSBOOK_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qwidget.h>
#include <qstring.h>

#include "kaddressbookiface.h"

class ContactEntry;
class ContactEntryList;
class ViewManager;
class PrefsDialog;
class LDAPSearchDialogImpl;

namespace KABC { class DistributionListEditor; }

/**
 * This class serves as the main window for KAddressBook.  It handles the
 * menus, toolbars, and status bars as well as creating the view.
 *
 * Most of the actions of the application will be resolved here or in the
 * view manager. Since the application can have multiple views, the actions 
 * cannot be connect directly to the views.
 *
 * This class should be used as the main widget of the application. It will
 * hold a VBox, with the view manager on top, and the quick edit widget on
 * the bottom.
 *
 * To interact with the view manager directly, it can be retrieved using
 * viewManager().
 *
 * @short Main window class
 * @author Don Sanders <dsanders@kde.org>
 * @version 0.1
 */
class KAddressBook : public QWidget
{
    Q_OBJECT
  public:
    KAddressBook( QWidget *parent, const char *name=0 );
    virtual ~KAddressBook();
    
    ViewManager *viewManager() { return mViewManager; }
    
  public slots:
    /** DCOP METHODS. NEED TO BE REMOVED!!! -mpilone */
    void addEmail( QString addr );
    void newContact();
    QStringList getKeys() const;
    QDict<ContactEntry> getEntryDict() const;
    void changeEntry( QString key, ContactEntry changeEntry);
    void removeEntry( QString key );
    void addEntry( ContactEntry newEntry );
    ASYNC showContactEditor( QString uid );    
    QString getNameByPhone( QString phone );
    /** END DCOP METHODS */
    
    /** Displays the distribution list editor dialog.
    *
    * This method name needs to be updated to something better.
    * -mpilone
    */
    void slotDistributionList();
    
    /** Saves the contents of the AddressBook back to disk.
    */
    void save();
    
    /** Reads the config file.
    */
    void readConfig();
    
    /** Writes the config file.
    */
    void writeConfig();
    
    /** Undo the last command using the undo stack.
    */
    void undo();
    
    /** Redo the last command that was undone, using the redo stack.
    */
    void redo();
    
    /** Import libkab data 
    */
    void importKDE2();
    
    /** Import comma-seperated list of all addressbook entries. 
    */
    void importCSV();
    
    /** Import VCard 2.1 files
    */
    void importVCard();
    
    /** Export comma-seperated list of all addressbook entries. 
    */
    void exportCSV();

    /** Shows the edit dialog for the given uid. If the uid is QString::null,
    *  the method will try to find a selected addressee in the view.
    */
    void editAddressee(QString uid = QString::null);
    
    /** Creates a new addressee and shows the edit dialog for it. If the
    * new edit dialog is cancel, the addressee will be destroyed. If the
    * edit dialog is accepted, the addressee will be inserted into the view
    * and the view will be told to refresh.
    */
    void newAddressee();
 
    /**
      Launches the configuration dialog.
    */
    void configure();
    
  protected slots:
    /** Called whenever the user selects an entry in the view.
    */
    void addresseeSelected(const QString &uid);
    
    /** called whenever the user activates an entry in the view.
    */
    void addresseeExecuted(const QString &uid);
    
    /** Called whenever the view is modified in some way. This could
    * mean that the view supports inline editing and the user is
    * editing a contact.
    */
    void viewModified();

    void slotOpenLDAPDialog();
  
  signals:
    /** Emitted whenever an addressee is selected in the view.
    *
    * @param selected True if an addressee was selected, false otherwise.
    */
    void addresseeSelected(bool selected);
    
    /** Emitted whenever the address book is modified in some way.
    *
    * @param mod True if the address book has been modified, false otherwise.
    */
    void modified(bool mod);
    
  private:   
    /** Updates an entry in the document to match the parameter.
    *
    * @param entryKey The entryKey of the addressee to modify. If this
    * entryKey is not found in the document, it is assumed that the
    * ContactEntry is new.
    *
    * @param ce The contact entry to copy the information into.
    */
    void updateAddressee(QString entryKey, ContactEntry *ce);
  
    /** Looks up the contact using the email address addr, then sets the name
    * to name. Proceeds to display the contact editor for the discovered
    * contact. -- Carry over for DCOP support.
    */
    virtual void updateContact( QString addr, QString name );
  
    ContactEntryList *mDocument;
    KABC::DistributionListEditor *mDistEditor;
    PrefsDialog *mPrefsDialog;
    LDAPSearchDialogImpl *mLdapSearchDialog;
    ViewManager *mViewManager;
};

#endif // KADDRESSBOOK_H
